# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.help_files import helps

#pylint: disable=line-too-long

helps['acr'] = """
            type: group
            short-summary: Commands to manage Azure container registries.
            """

helps['acr credential'] = """
            type: group
            short-summary: Manage administrator login credentials for Azure container registries.
            """

helps['acr repository'] = """
            type: group
            short-summary: Manage repositories for Azure container registries.
            """

helps['acr list'] = """
            type: command
            examples:
                - name: List container registries and show result in a table.
                  text:
                    az acr list -o table
                - name: List container registries in a resource group and show result in a table.
                  text:
                    az acr list -g myResourceGroup -o table
            """

helps['acr create'] = """
            type: command
            examples:
                - name: Create a container registry with a new storage account.
                  text:
                    az acr create -n myRegistry -g myResourceGroup -l southcentralus
                - name: Create a container registry with an existing storage account.
                  text:
                    az acr create -n myRegistry -g myResourceGroup -l southcentralus --storage-account-name myStorageAccount
            """

helps['acr update'] = """
            type: command
            short-summary: Updates a container registry.
            examples:
                - name: Update tags for a container registry
                  text:
                    az acr update -n myRegistry --tags key1=value1 key2=value2
                - name: Update storage account for a container registry
                  text:
                    az acr update -n myRegistry --storage-account-name myStorageAccount
                - name: Enable admin user for a container registry
                  text:
                    az acr update -n myRegistry --admin-enabled true
            """

helps['acr repository list'] = """
            type: command
            examples:
                - name: List repositories in a given container registry if admin user is enabled
                  text:
                    az acr repository list -n myRegistry
                - name: List repositories in a given container registry with credentials
                  text:
                    az acr repository list -n myRegistry -u myUsername -p myPassword
            """

helps['acr repository show-tags'] = """
            type: command
            examples:
                - name: Show tags of a given repository in a given container registry if admin user is enabled
                  text:
                    az acr repository show-tags -n myRegistry --repository myRepository
                - name: Show tags of a given repository in a given container registry with credentials
                  text:
                    az acr repository show-tags -n myRegistry --repository myRepository -u myUsername -p myPassword
            """



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core._util import CLIError
from ._factory import get_acr_service_client
from ._utils import get_registry_by_name

import azure.cli.core.azlogging as azlogging
logger = azlogging.get_az_logger(__name__)

def acr_credential_show(registry_name, resource_group_name=None):
    '''Gets the administrator login credentials for the specified container registry.
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    '''
    registry, resource_group_name = get_registry_by_name(registry_name, resource_group_name)
    client = get_acr_service_client().registries

    if registry.admin_user_enabled: #pylint: disable=no-member
        return client.get_credentials(resource_group_name, registry_name)

    admin_not_enabled_error(registry_name)

def acr_credential_renew(registry_name, resource_group_name=None):
    '''Regenerates the administrator login credentials for the specified container registry.
    :param str registry_name: The name of container registry
    :param str resource_group_name: The name of resource group
    '''
    registry, resource_group_name = get_registry_by_name(registry_name, resource_group_name)
    client = get_acr_service_client().registries

    if registry.admin_user_enabled: #pylint: disable=no-member
        return client.regenerate_credentials(resource_group_name, registry_name)

    admin_not_enabled_error(registry_name)

def admin_not_enabled_error(registry_name):
    raise CLIError("Run 'az acr update -n {} --admin-enabled true' to enable admin first.".format(
        registry_name))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core._config import az_config
from azure.mgmt.resource.resources import ResourceManagementClient
from azure.mgmt.storage import StorageManagementClient
from azure.mgmt.containerregistry import ContainerRegistryManagementClient

from azure.cli.core.commands.client_factory import get_mgmt_service_client

import azure.cli.core.azlogging as azlogging
logger = azlogging.get_az_logger(__name__)

def get_arm_service_client():
    '''Returns the client for managing ARM resources.
    '''
    return get_mgmt_service_client(ResourceManagementClient)

def get_storage_service_client():
    '''Returns the client for managing storage accounts.
    '''
    return get_mgmt_service_client(StorageManagementClient)

def get_acr_service_client():
    '''Returns the client for managing container registries.
    '''
    customized_api_version = get_acr_api_version()
    if customized_api_version:
        return get_mgmt_service_client(ContainerRegistryManagementClient,
                                       api_version=customized_api_version)
    else:
        return get_mgmt_service_client(ContainerRegistryManagementClient)

def get_acr_api_version():
    '''Returns the api version for container registry
    '''
    customized_api_version = az_config.get('acr', 'apiversion', None)
    if customized_api_version:
        logger.warning('Customized api-version is used: %s', customized_api_version)
    return customized_api_version



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from collections import OrderedDict

from ._utils import get_resource_group_name_by_resource_id

_registry_map = {
    'name': 'NAME',
    'resourceGroup': 'RESOURCE GROUP',
    'location': 'LOCATION',
    'loginServer': 'LOGIN SERVER',
    'creationDate': 'CREATION DATE',
    'adminUserEnabled': 'ADMIN ENABLED'
}

_admin_user_map = {
    'username': 'USERNAME',
    'password': 'PASSWORD'
}

_order_map = {
    'NAME': 1,
    'RESOURCE GROUP': 2,
    'LOCATION': 3,
    'LOGIN SERVER': 11,
    'CREATION DATE': 12,
    'ADMIN ENABLED': 13,
    'USERNAME': 31,
    'PASSWORD': 32
}

def output_format(result):
    '''Returns the list of container registries each of which is an ordered dictionary.
    :param list/dict result: The (list of) container registry object(s)
    '''
    obj_list = result if isinstance(result, list) else [result]
    return [_format_group(item) for item in obj_list]

def _format_group(item):
    '''Returns an ordered dictionary of the container registry.
    :param dict item: The container registry object
    '''
    registry_info = {_registry_map[key]: str(item[key]) for key in item if key in _registry_map}

    if 'id' in item and item['id']:
        resource_group_name = get_resource_group_name_by_resource_id(item['id'])
        registry_info['RESOURCE GROUP'] = resource_group_name

    admin_user_info = {_admin_user_map[key]: str(item[key])
                       for key in item if key in _admin_user_map}

    all_info = registry_info.copy()
    all_info.update(admin_user_info)

    return OrderedDict(sorted(all_info.items(), key=lambda t: _order_map[t[0]]))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core._util import CLIError

from ._factory import (
    get_arm_service_client,
    get_acr_service_client
)

import azure.cli.core.azlogging as azlogging
logger = azlogging.get_az_logger(__name__)

def validate_registry_name(namespace):
    if namespace.registry_name:
        client = get_acr_service_client().registries
        registry_name = namespace.registry_name

        result = client.check_name_availability(registry_name)

        if not result.name_available: #pylint: disable=no-member
            raise CLIError(result.message) #pylint: disable=no-member

def validate_resource_group_name(namespace):
    if namespace.resource_group_name:
        client = get_arm_service_client()
        resource_group_name = namespace.resource_group_name

        if not client.resource_groups.check_existence(resource_group_name):
            logger.warning('Command to create a resource group:')
            logger.warning('  az group create -n <name> -l <location>')
            raise CLIError(
                'The resource group {} does not exist in the current subscription.'\
                .format(resource_group_name))
